@@ -1,9 +1,11 @@
 """
 External SVM classifiers using a command-line interface
 TODO: Why does SVM using 'output probability' perform different from without?
+TODO: Use pexpect instead of popen, tie in with progressbar output
 """
 from hydrat.classifier.abstract import Learner, Classifier, NotInstalledError
 from hydrat import config
+from hydrat.configuration import Configurable, EXE
 
 from itertools import izip
 
@@ -60,7 +62,7 @@ class SVMLearner(Learner):
   def __init__(self):
     self.range_path = None
     self.model_path = None
-    self.clear_temp = False #TODO 
+    self.clear_temp = config.getboolean('debug', 'clear_temp_files')
     Learner.__init__(self)
 
   def _check_installed(self):
@@ -151,7 +153,7 @@ class SVMClassifier(Classifier):
     self.scaler      = scaler
     self.range_path  = range_path
     self.num_classes = num_classes
-    self.clear_temp  = True # Clear temp files after execution
+    self.clear_temp  = config.getboolean('debug','clear_temp_files')
 
   def __invoke_classifier(self, test_path):
     #Create a temporary file for the results
@@ -211,6 +213,8 @@ class SVMClassifier(Classifier):
       # Set the first class from what we earlier extracted.
       classifications[0, int(first_line)] = True
       for doc_index, l in enumerate(result_file):
+        # TODO: Work out libSVM splits out class labels that are clearly out of the range
+        #       of possible values. Seen it happen in multiclass contexts.
         class_index = int(l)
         classifications[doc_index+1, class_index] = True
 
@@ -241,7 +245,7 @@ class SVMClassifier(Classifier):
     classifications = self.__parse_result(result_path, num_test_docs)
     return classifications
 
-class libsvmExtL(SVMLearner):
+class libsvmExtL(Configurable, SVMLearner):
   """
   -s svm_type : set type of SVM (default 0)
           0 -- C-SVC
@@ -271,9 +275,9 @@ class libsvmExtL(SVMLearner):
   """
   __name__ = 'libsvm_ext'
   requires =\
-    { 'libsvmlearner' : 'svm-train' 
-    , 'libsvmclassifier' : 'svm-predict'
-    , 'libsvmscaler' : 'svm-scale'
+    { ('tools', 'libsvmlearner')    : EXE('svm-train')
+    , ('tools','libsvmclassifier')  : EXE('svm-predict')
+    , ('tools','libsvmscaler')      : EXE('svm-scale')
     }
 
   def _check_installed(self):
@@ -308,7 +312,8 @@ class libsvmExtL(SVMLearner):
     p.update(self.__params)
     return p
 
-class bsvmL(SVMLearner):
+# TODO: Deal with mutliclass fed to BSVM. Seems to choke, probably some error output on stderr.
+class bsvmL(Configurable, SVMLearner):
   """
   -s svm_type : set type of SVM (default 0)
           0 -- multi-class bound-constrained support vector classification (SVC)
@@ -337,8 +342,8 @@ class bsvmL(SVMLearner):
   """
   __name__ = 'bsvm'
   requires =\
-    { 'bsvmlearner' : 'bsvm-train' 
-    , 'bsvmclassifier' : 'bsvm-predict'
+    { ('tools','bsvmlearner')    : EXE('bsvm-train')
+    , ('tools','bsvmclassifier') : EXE('bsvm-predict')
     }
 
 
